-module(akhamoth).

-export([
    compile_file/1,
    compile_binary/1
]).

compile_file(Path) ->
    {ok, Source} = file:read_file(Path),
    compile_binary(Source).

compile_binary(Source) ->
    Lx = akh_lex:new(Source),
    {Tokens, SourceMap} = collect(Lx, []),
    lists:map(fun(T) -> location(T, SourceMap) end, Tokens).

collect(Lx, Acc) ->
    case akh_lex:next(Lx) of
        none -> {Acc, akh_lex:source_map(Lx)};
        {ok, T, L} -> collect(L, [T | Acc])
    end.

location({T, O, _}, SourceMap) ->
    {T, akh_source_map:location(O, SourceMap)};
location({T, O}, SourceMap) ->
    {T, akh_source_map:location(O, SourceMap)}.
